<?php
namespace Blesta\Core\Automation\Tasks\Task;

use Blesta\Core\Automation\Tasks\Common\AbstractTask;
use Blesta\Core\Automation\Type\Common\AutomationTypeInterface;
use Configure;
use Language;
use Loader;
use stdClass;

/**
 * The cleanup_logs automation task
 *
 * @package blesta
 * @subpackage blesta.core.Automation.Tasks.Task
 * @copyright Copyright (c) 2018, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class CleanupLogs extends AbstractTask
{
    /**
     * Initialize a new task
     *
     * @param AutomationTypeInterface $task The raw automation task
     * @param array $options An additional options necessary for the task
     *  - print_log True to print logged content to stdout, or false otherwise (default false)
     *  - cli True if this is being run via the Command-Line Interface, or false otherwise (default true)
     */
    public function __construct(AutomationTypeInterface $task, array $options = [])
    {
        parent::__construct($task, $options);

        Loader::loadModels($this, ['Logs']);
        Loader::loadComponents($this, ['SettingsCollection']);
    }

    /**
     * {@inheritdoc}
     */
    public function run()
    {
        // This task cannot be run right now
        if (!$this->isTimeToRun()) {
            return;
        }

        // Log the task has begun
        $this->log(Language::_('Automation.task.cleanup_logs.attempt', true));

        // Execute the clean-up logs cron task
        $this->process($this->task->raw());

        // Log the task has completed
        $this->log(Language::_('Automation.task.cleanup_logs.completed', true));
        $this->logComplete();
    }

    /**
     * Processes the task
     *
     * @param stdClass $data The cleanup logs task data
     */
    private function process(stdClass $data)
    {
        // Get the date at which logs should be purged from the settings
        $company_settings = $this->SettingsCollection->fetchSettings(null, $data->company_id);

        // Run the cron task if enabled and the rotation policy (log_days) is valid
        if (is_numeric($company_settings['log_days'])) {
            // Get the (local) date at which logs should be purged
            $start_date = $this->date->format('c');
            $past_date = $this->date->modify(
                $start_date,
                '-' . abs((int)$company_settings['log_days']) . ' days',
                'Y-m-d H:i:s',
                $company_settings['timezone']
            );

            // Delete the gateway logs
            if (Configure::get('Blesta.auto_delete_gateway_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_gateway_deleted',
                        true,
                        $this->Logs->deleteGatewayLogs($past_date)
                    )
                );
            }

            // Delete the module logs
            if (Configure::get('Blesta.auto_delete_module_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_module_deleted',
                        true,
                        $this->Logs->deleteModuleLogs($past_date)
                    )
                );
            }

            // Delete the account access logs
            if (Configure::get('Blesta.auto_delete_accountaccess_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_accountaccess_deleted',
                        true,
                        $this->Logs->deleteAccountAccessLogs($past_date)
                    )
                );
            }

            // Delete the contact logs
            if (Configure::get('Blesta.auto_delete_contact_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_contact_deleted',
                        true,
                        $this->Logs->deleteContactLogs($past_date)
                    )
                );
            }

            // Delete the client setting logs
            if (Configure::get('Blesta.auto_delete_client_setting_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_client_settings_deleted',
                        true,
                        $this->Logs->deleteClientSettingLogs($past_date)
                    )
                );
            }

            // Delete the email logs
            if (Configure::get('Blesta.auto_delete_email_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_email_deleted',
                        true,
                        $this->Logs->deleteEmailLogs($past_date)
                    )
                );
            }

            // Delete the user logs
            if (Configure::get('Blesta.auto_delete_user_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_user_deleted',
                        true,
                        $this->Logs->deleteUserLogs($past_date)
                    )
                );
            }

            // Delete the service logs
            if (Configure::get('Blesta.auto_delete_service_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_service_deleted',
                        true,
                        $this->Logs->deleteServiceLogs($past_date)
                    )
                );
            }

            // Delete the transaction logs
            if (Configure::get('Blesta.auto_delete_transaction_logs')) {
                $this->log(
                    Language::_(
                        'Automation.task.cleanup_logs.logs_transaction_deleted',
                        true,
                        $this->Logs->deleteTransactionLogs($past_date)
                    )
                );
            }
        }
    }

    /**
     * {@inheritdoc}
     */
    protected function isTimeToRun()
    {
        return $this->task->canRun(date('c'));
    }
}
